
float Timer : TIME < string UIWidget="None"; >;

//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/* Ambientes Licht */
float3 foreColor <
	string UIName =  "Foreground Color";
	string UIWidget = "Color";
> = {0.9f, 1.0f, 0.5f};

/* Ambientes Licht */
float3 backColor <
	string UIName =  "Background Color";
	string UIWidget = "Color";
> = {0.5f, 0.5f, 0.1f};

float baseAmplitude <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.1;
	string UIName =  "Base Amplitude";
> = 0.6;

float baseFrequency <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.1;
	string UIName =  "Base Amplitude";
> = 0.1;

float animSpeed <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 10.0;
	float  UIStep = 0.1;
	string UIName =  "Animation Speed";
> = 2.0;

float maxDisp <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 0.5;
	float  UIStep = 0.01;
	string UIName =  "Maximum Displacement";
> = 0.05;


float perturbU <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 20.0;
	float  UIStep = 0.1;
	string UIName =  "Perturbation U";
> = 13.05;

float perturbV <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 20.0;
	float  UIStep = 0.1;
	string UIName =  "Perturbation U";
> = 15.1;

//////// COLOR & TEXTURE /////////////////////

texture NoiseTexture : DIFFUSE <
    string ResourceName = "noise.dds";
    string UIName =  "Noise Texture";
    string ResourceType = "2D";
>;

sampler2D NoiseSampler = sampler_state {
    Texture = <NoiseTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Mirror;
    AddressV = Mirror;
};  

// shared shadow mapping supported in Cg version

//////// CONNECTOR DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 LightVec	    : TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldTangent	: TEXCOORD3;
    float3 WorldBinormal: TEXCOORD4;
    float3 WorldView	: TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent = mul(IN.Tangent,WorldITXf).xyz;
    OUT.WorldBinormal = mul(IN.Binormal,WorldITXf).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,WorldXf).xyz;
    OUT.UV = IN.UV.xy;
    OUT.WorldView = normalize(ViewIXf[3].xyz - Pw);
    OUT.HPosition = mul(Po,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 std_PS(vertexOutput IN) : COLOR {
	
	float2 uv = IN.UV;
	uv.x += maxDisp*sin(animSpeed*Timer+perturbU*IN.UV.y);
	uv.y += maxDisp*cos(animSpeed*Timer+perturbV*IN.UV.x);
	float f = 0;
	for(int i = 0; i < 20; ++i) {
    	f+= baseAmplitude * tex2D(NoiseSampler,baseFrequency *uv).rgb;
		
		baseAmplitude /= 2.0;
		baseFrequency *= 2.0;

	}
		
	float3 result = lerp(foreColor, backColor, f);
    return float4(result,1);
}

///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 Main10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, std_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


technique Main <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        VertexShader = compile vs_2_0 std_VS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		AlphaBlendEnable = false;
		CullMode = None;
        PixelShader = compile ps_2_a std_PS();
    }
}

/////////////////////////////////////// eof //
